@@ -1,6 +1,7 @@
 create.exponential.basis <- function (rangeval=c(0,1), nbasis=NULL,
                          ratevec=NULL, dropind=NULL, quadvals=NULL,
-                         values=NULL, basisvalues=NULL, names='exp')
+                         values=NULL, basisvalues=NULL, names='exp',
+                         axes=NULL)
 {
 
 #  This function creates an exponential functional data basis
@@ -173,9 +174,13 @@ create.exponential.basis <- function (rangeval=c(0,1), nbasis=NULL,
       if(length(names)>1)
         stop('length(names) = ', length(names), ';  must be either ',
              '1 or nbasis = ', nbasis)
-      paste(names, 0:(nbasis-1), sep="")
+      basisobj$names <- paste(names, 0:(nbasis-1), sep="")
     }
   }
+##
+## 6.  Done
+##
+  if(!is.null(axes))basisobj$axes <- axes
 
   basisobj
 
