@@ -1,11 +1,12 @@
-eval.bifd <- function(sevalarg, tevalarg, bifd, sLfdobj = 0, tLfdobj = 0) {
+eval.bifd <- function(sevalarg, tevalarg, bifd, sLfdobj = 0, tLfdobj = 0, 
+                            returnMatrix=FALSE) {
 
   #  Evaluates a bi-functional data object BIFD at argument values in arrays
   #  SEVALARG and TEVALARG.  Differential operators SLFD and TLFD are
   #     applied to BIFD if present.
 
-  #  Last modified 2007 by Spencer(?); previous mod 26 October 2005
-#  fixed references to nonexistent sLfd, tLfd
+  #  Last modified 7 May 2012 by Jim Ramsay
+
   if (!is.vector(sevalarg)) stop(
      "First argument is not a vector.")
   if (!is.vector(tevalarg)) stop(
@@ -38,33 +39,9 @@ eval.bifd <- function(sevalarg, tevalarg, bifd, sLfdobj = 0, tLfdobj = 0) {
   snderiv <- sLfdobj$nderiv
   tnderiv <- tLfdobj$nderiv
 
-#  sbasismat <- getbasismatrix(sevalarg, sbasisobj, snderiv)
-#  if (snderiv > 0 && !is.null(sLfdobj)) {
-#    sLfdmat <- eval.fd(sevalarg, sLfdobj)
-#    onerow  <- rep(1,snbasis)
-#    for (j in 1:snderiv) {
-#      if (any(abs(sLfdmat[,j])) > 1e-7) {
-#        sbasismat <- sbasismat + outer(sLfdmat[,j],onerow)*
-#                         getbasismatrix(sevalarg, sbasisobj, j-1)
-#      }
-#    }
-#  }
+  sbasismat <- eval.basis(sevalarg,sbasisobj,sLfdobj,returnMatrix)
 
-  sbasismat <- eval.basis(sevalarg,sbasisobj,sLfdobj)
-
-#  tbasismat <- getbasismatrix(tevalarg, tbasisobj, tnderiv)
-#  if (tnderiv > 0 && !is.null(tLfdobj)) {
-#    tLfdmat <- eval.fd(tevalarg, tLfdobj)
-#    onerow <- rep(1,tnbasis)
-#    for (j in 1:tnderiv) {
-#      if (any(abs(tLfdmat[,j])) > 1e-7) {
-#        tbasismat <- tbasismat + outer(tLfdmat[,j],onerow)*
-#                         getbasismatrix(tevalarg, tbasisobj, j-1)
-#      }
-#    }
-#  }
-
-  tbasismat <- eval.basis(tevalarg,tbasisobj,tLfdobj)
+  tbasismat <- eval.basis(tevalarg,tbasisobj,tLfdobj,returnMatrix)
 
   if (ndim == 2) {
     evalbifd <- sbasismat %*% coef %*% t(tbasismat)
