@@ -1,4 +1,5 @@
-fRegress.CV <- function(y, xfdlist, betalist, wt=NULL, CVobs=1:N, ...)
+fRegress.CV <- function(y, xfdlist, betalist, wt=NULL, CVobs=1:N,
+                        returnMatrix=FALSE, ...)
 {
 
 # FREGRESS.CV computes cross-validated error sum of squares
@@ -6,13 +7,13 @@ fRegress.CV <- function(y, xfdlist, betalist, wt=NULL, CVobs=1:N, ...)
 # generalized cross validation scores are now returned by fRegress
 # when scalar responses are used.
 
-# last modified June 21 2010 by Giles Hooker
+# last modified 8 May 2012 by Jim Ramsay
 
 argList  <- fRegressArgCheck(y, xfdlist, betalist, wt)
 yfdPar   <- argList$yfdPar
 xfdlist  <- argList$xfdlist
 betalist <- argList$betalist
-wt       <- argList$wt 
+wt       <- argList$wt
 
 p <- length(xfdlist)
 N <- dim(xfdlist[[1]]$coef)[2]
@@ -25,7 +26,7 @@ if (inherits(yfdPar, "numeric"))  {
     SSE.CV <- 0
     errfd  <- c()
     for (m in 1:M) {
-      i        <- CVobs[m]  
+      i        <- CVobs[m]
       xfdlisti <- vector("list",p)
       for (j in 1:p) {
         xfdj          <- xfdlist[[j]]
@@ -57,8 +58,8 @@ if (inherits(yfdPar, "numeric"))  {
         nfine      <- max(501, bbasisj$nbasis*10+1)
         tfine      <- seq(rangej[1], rangej[2], len=nfine)
         delta      <- tfine[2]-tfine[1]
-        betavec    <- eval.fd(tfine, betafdj)
-        xveci      <- eval.fd(tfine, xfdj[i])
+        betavec    <- eval.fd(tfine, betafdj, 0, returnMatrix)
+        xveci      <- eval.fd(tfine, xfdj[i], 0, returnMatrix)
         yhati      <- yhati + delta*(sum(xveci*betavec) -
                                     0.5*( xveci[1]    *betavec[1] +
                                           xveci[nfine]*betavec[nfine] ))
@@ -66,19 +67,20 @@ if (inherits(yfdPar, "numeric"))  {
       errfd[i] = yvec[i] - yhati;
       SSE.CV <- SSE.CV + errfd[i]^2
     }
- } else { 
+ } else {
     yfd      <- yfdPar$fd
     SSE.CV   <- 0
     errcoefs <- c()
     for(m in 1:N){
+#        cat(m, " ")
       i <-  CVobs[m]
-      txfdlist <- xfdlist           
+      txfdlist <- xfdlist
       for(k in 1:p){
         txfdlist[[k]] <- xfdlist[[k]][-i]
       }
       wti = wt[-i]
       tres <- fRegress(yfd[-i],txfdlist,betalist,wti)
-      yhat <- 0                       
+      yhat <- 0
       for(k in 1:p){
         yhat <- yhat + xfdlist[[k]][i]*tres$betaestlist[[k]]$fd
       }
