@@ -1,5 +1,5 @@
 fRegress <- function(y, xfdlist, betalist, wt=NULL, 
-                     y2cMap=NULL, SigmaE=NULL) 
+                     y2cMap=NULL, SigmaE=NULL, returnMatrix=FALSE) 
 {
 
 #  FREGRESS  Fits a functional linear model using multiple
@@ -37,6 +37,9 @@ fRegress <- function(y, xfdlist, betalist, wt=NULL,
 #  SIGMAE   ... Estimate of the covariances among the residuals.  This
 #               can only be estimated after a preliminary analysis
 #               with FREGRESS.
+#  RETURNMATRIX ... If False, a matrix in sparse storage model can be returned
+#               from a call to function BsplineS.  See this function for
+#               enabling this option.
 #
 #  Returns FREGRESSLIST  ... A list containing seven members with names:
 #    yfdPar      ... first  argument of FREGRESS
@@ -53,7 +56,7 @@ fRegress <- function(y, xfdlist, betalist, wt=NULL,
 #    wt          ... weights for observations
 #    df          ... degrees of freedom for fit
 
-#  Last modified 3 February 2010 by Jim Ramsay
+#  Last modified 8 May 2012 by Jim Ramsay
 
 arglist <- fRegressArgCheck(y, xfdlist, betalist, wt)
 
@@ -208,10 +211,10 @@ if (inherits(yfdPar, "fdPar")) {
     yhatmat <- matrix(0,nfine,N)
     for (j in 1:p) {
 	  xfdj       <- xfdlist[[j]]
-        xmatj      <- eval.fd(tfine, xfdj)
+        xmatj      <- eval.fd(tfine, xfdj, 0, returnMatrix)
         betafdParj <- betaestlist[[j]]
         betafdj    <- betafdParj$fd
-        betavecj   <- eval.fd(tfine, betafdj)
+        betavecj   <- eval.fd(tfine, betafdj, 0, returnMatrix)
         yhatmat    <- yhatmat + xmatj*as.vector(betavecj)
     }
     yhatfdobj <- smooth.basis(tfine, yhatmat, ybasisobj)$fd
@@ -232,7 +235,7 @@ if (inherits(yfdPar, "fdPar")) {
              stop("Dimensions of Y2CMAP not correct.")
         }
 
-        ybasismat = eval.basis(tfine, ybasisobj)
+        ybasismat = eval.basis(tfine, ybasisobj, 0, returnMatrix)
 
         deltat    = tfine[2] - tfine[1]
 
@@ -245,9 +248,9 @@ if (inherits(yfdPar, "fdPar")) {
             betafdParj = betalist[[j]]
             betabasisj = betafdParj$fd$basis
             ncoefj     = betabasisj$nbasis
-            bbasismatj = eval.basis(tfine, betabasisj)
+            bbasismatj = eval.basis(tfine, betabasisj, 0, returnMatrix)
             xfdj       = xfdlist[[j]]
-            tempj      = eval.fd(tfine, xfdj)
+            tempj      = eval.fd(tfine, xfdj, 0, returnMatrix)
             #  row indices of BASISPRODMAT to fill
             mj1    = mj2 + 1
             mj2    = mj2 + ncoefj
@@ -281,7 +284,7 @@ if (inherits(yfdPar, "fdPar")) {
             mj1 	     = mj2 + 1
             mj2 	     = mj2 + ncoefj
             indexj     = mj1:mj2
-            bbasismat  = eval.basis(tfine, betabasisj)
+            bbasismat  = eval.basis(tfine, betabasisj, 0, returnMatrix)
             bvarj      = bvar[indexj,indexj]
             bstderrj   = sqrt(diag(bbasismat %*% bvarj %*% t(bbasismat)))
             bstderrfdj = smooth.basis(tfine, bstderrj, betabasisj)$fd
@@ -420,10 +423,10 @@ if (inherits(yfdPar, "fdPar")) {
             nfine      <- max(501,10*xnbasis+1)
             tfine      <- seq(xrng[1], xrng[2], len=nfine)
             deltat     <- tfine[2]-tfine[1]
-            xmat       <- eval.fd(tfine, xfdj)
+            xmat       <- eval.fd(tfine, xfdj, 0, returnMatrix)
             betafdParj <- betaestlist[[j]]
             betafdj    <- betafdParj$fd
-            betamat    <- eval.fd(tfine, betafdj)
+            betamat    <- eval.fd(tfine, betafdj, 0, returnMatrix)
             fitj       <- deltat*(crossprod(xmat,betamat) -
 						                 0.5*(outer(xmat[1,    ],betamat[1,    ]) +
 				                          outer(xmat[nfine,],betamat[nfine,])))
@@ -469,7 +472,7 @@ if (inherits(yfdPar, "fdPar")) {
             betarng    <- betabasisj$rangeval
             nfine      <- max(c(501,10*ncoefj+1))
             tfine      <- seq(betarng[1], betarng[2], len=nfine)
-            bbasismat  <- eval.basis(tfine, betabasisj)
+            bbasismat  <- eval.basis(tfine, betabasisj, 0, returnMatrix)
             bstderrj   <- sqrt(diag(bbasismat %*% bvarj %*% t(bbasismat)))
             bstderrfdj <- smooth.basis(tfine, bstderrj, betabasisj)$fd
             betastderrlist[[j]] <- bstderrfdj
