@@ -1,17 +1,29 @@
-function plot_FEM(fdobj, X, Y)
+function plot_FEM(fdobj, X, Y, nderivs, nfine)
 % PLOT  Plots a FEM object FDOBJ over a rectangular grid defined by 
 % vectors X and Y;
 %
-% Last modified on 21 August 2010
+% Last modified on 15 May 2011
 
-if nargin < 3,  Y = [];  end
-if nargin < 2,  X = [];  end
+if nargin < 5,                      nfine = 101;      end
+if nargin < 4 || isempty(nderivs),  nderivs = [0,0];  end
+if nargin < 3,                      Y = [];           end
+if nargin < 2,                      X = [];           end
 
 if ~isa_fd(fdobj)
    error('FDOBJ is not an FD object')
 end
 
+%  check derivatives
+
+if length(nderivs) ~= 2
+    error('NDERIVS not of length 2.');
+end
+if sum(nderivs) > 2
+    error('Maximum derivative order is greater than two.');
+end
+
 coefmat = full(getcoef(fdobj));
+nsurf = size(coefmat,2);
 
 basisobj = getbasis(fdobj);
 
@@ -21,7 +33,6 @@ p         = params.p;
 t         = params.t;
 t         = t(:,1:3);
 
-nfine = 101;
 if isempty(X)
     xmin = min(p(:,1));
     xmax = max(p(:,1));
@@ -45,9 +56,12 @@ Ymat = ones(nx,1)*Y';
 Xvec = Xmat(:);
 Yvec = Ymat(:);
 
-evalmat = eval_FEM_fd(Xvec, Yvec, fdobj);
 
-nsurf = size(coefmat,2);
+evalmat = zeros(nx*ny,nsurf);
+for iopr = 1:size(nderivs,1)
+    evalmat = evalmat + eval_FEM_fd(Xvec, Yvec, fdobj, nderivs(iopr,:));
+end
+
 for isurf=1:nsurf
     evalmati = reshape(evalmat(:,isurf),nx,ny)';
     surf(X,Y,evalmati);
